----------------------------------
-- Composant ALU               	--
-- version prototype simplifi	-- 
--				--
-- auteur : Matthias Colin      --
-- date   : 6/05/2009           --
--                              --
----------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use vonneuman2010.pack_Von_Neumann.all;


-- dclaration de l'entit alu
entity alu is
port(
	-- les entres
	a : in t_mot; -- 1re oprande
	b : in t_mot; -- 2me oprande
	op : in t_operation; -- opration  excuter sur les oprandes
	enable : in std_logic; -- signal activation de l'alu  niveau bas
	clock : in std_logic; -- signal d'horloge active sur front montant
	reset : in std_logic; -- signal de rinitialisation asynchrone
	
	-- les sorties
	s : out t_mot; -- rsultat
	flag_c : out std_logic; -- flag d'tat sur la retenue
	flag_z : out std_logic 	-- flag d'tat sur un rsultat nul
);
end alu;

-- architecture du composant ALU
architecture alu_arch_proto of alu is
	-- mmorisation interne des FLAGS
	signal flag_c_interne, flag_z_interne : std_logic;
begin

	-- gestion synchrone du traitelent des oprations
	process (clock, reset)
	variable produit : std_logic_vector(7 downto 0);
	variable somme : std_logic_vector(4 downto 0);
	variable aa, bb, moduloo : integer range 0 to 15;
	begin
		-- initialisation asynchrone des commandes
		if (reset = '0') then
			-- rinitialisation des flags, de la sortie et de la mmorisation d'autorisation
			flag_c_interne <= '0';
			flag_z_interne <= '0';
			s <= (others => '0');
		-- comportement synchrone de l'ALU sur front montant d'horloge		
		elsif (clock'event and clock = '1') then
			-- si le signal enable est inactif (niveau bas)
			if ( enable = '0' ) then
				case op is
					when ADD =>
						-- somme de 2 std_logic unsigned sur 5 bits
						-- pour garder la retenue
						somme := ("0"&a) + ("0"&b);
						-- rsultat obtenu sur les 4 bits de poids faible
						s <= somme(3 downto 0);
						-- retenue si bit de poids 4  '1'
						flag_c_interne <= somme(4) after 4 ns;
						-- rsultat nul si aucun '1' sur les 4 bits de poids faible
						flag_z_interne <=  ((somme(3) nor somme(2)) nor (somme(1) nor somme(0))) after 4 ns;
					when ADC =>
						-- somme de 2 std_logic unsigned sur 5 bits
						-- pour garder la retenue
						somme := ("0" & a) + ("0" & b) + ("000" & flag_c_interne);
						-- rsultat obtenu sur les 4 bits de poids faible
						s <= somme(3 downto 0);
						-- retenue si bit de poids 4  '1'
						flag_c_interne <= somme(4) after 4 ns;
						-- rsultat nul si aucun '1' sur les 4 bits de poids faible
						flag_z_interne <=  ((somme(3) nor somme(2)) nor (somme(1) nor somme(0))) after 4 ns;
					when SUB =>
						s <= a - b;
						if b > a then
							flag_c_interne <= '1';
						else
							flag_c_interne <= '0';
						end if;
						if b = a then
							flag_z_interne <= '1';
						else
							flag_z_interne <= '0';
						end if;
					when MULT =>
						-- multiplication de 2 std_logic unsigned sur 4 bits 
						-- donne 1 std_logic sur 8 bits
						produit := a * b; 
						-- rsultat obtenu sur les 4 bits de poids faible
						s <= produit(3 downto 0) after 4 ns;
						-- retenue si un bit  '1' sur les 4 bits de poids fort
						flag_c_interne <=  produit(7) or produit(6) or produit(5) or produit(4) after 4 ns;
						-- rsultat nul si aucun '1' sur les 4 bits de poids faible
						flag_z_interne <=  ((produit(3) nor produit(2)) nor (produit(1) nor produit(0))) after 4 ns;
					when DIV =>
						aa := conv_integer(a);
						bb := conv_integer(b);
						if bb = 0 then
							flag_z_interne <= 'U';
							flag_c_interne <= '1';
							s <= "UUUU";
						else
							s <= conv_std_logic_vector(aa/bb, 4);
							if (aa = 0) or (bb > aa) then
								flag_z_interne <= '1';
							else
								flag_z_interne <= '0';
							end if;
							flag_c_interne <= '0';
						end if;
					when MODULO =>
						aa := conv_integer(a);
						bb := conv_integer(b);
						if bb = 0 then
							flag_z_interne <= 'U';
							flag_c_interne <= '1';
							s <= "UUUU";
						else
							moduloo := aa mod bb; 
							s <= conv_std_logic_vector(moduloo, 4);
							if (moduloo = 0) then
								flag_z_interne <= '1';
							else
								flag_z_interne <= '0';
							end if;
							flag_c_interne <= '0';
						end if;
				end case;
			end if;	
		end if;
   	end process;
	
	-- connection des sorties de FLAGs
	flag_c <= flag_c_interne;
	flag_z <= flag_z_interne;	

end alu_arch_proto;























