--------------------------------------------------
-- package pack_cpu                   			--
-- auteur : Matthias Colin                    	--
-- date   : 6/05/2009                       	--
--                                 				--
-- dfinition des types et fonctions      		--
-- internes  l'unit de commande				--
-- composants de la machine de Von Neumann    	--
--------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use vonneuman2010.pack_Von_Neumann.all;

package pack_cpu is

-- type de tous les codes d'oprations d'instructions tenant sur 1 mot
-- et ceux sur 2 mots regroups en 2 familles suivant la valeur du 1er mot
type t_opcode1word is (O_READ, LD_TO_RJ, LD_FROM_RJ, LD_CSTE, 
	JRC, JRNC, JRZ, JRNZ, JR, DJNZ, OP2_FAMILY1, OP2_FAMILY2);

-- type de tous les codes d'oprations d'instructions tenant sur 2 mots
type t_opcode2word is (O_ADD, O_ADC, O_MINUS, O_MULT, O_DIV, 
	O_MOD, INC, O_WRITE, ERROR);

-- prototypes des mthodes

-- fonction decode_1st_word : dcode le code d'opration d'une instruction
-- 		 partir du 1er mot de celle-ci
-- paramtre mot : mot  dcoder
-- retour : le code du mot
function decode_1st_word(mot : in t_mot) return t_opcode1word;

-- fonction decode_1st_word : dcode le code d'opration d'une instruction
-- 		 partir du 2e mot de celle-ci et du rsultat du dcodage du 1er mot
-- paramtre opcode1 : code correspondant  la 1re partie du dcodage (OP2_FAMILY1 ou
-- 		OP2_FAMILY2)
-- paramtre mot : mot  dcoder
-- retour : le code du mot
function decode_2nd_word(opcode1 : in t_opcode1word; mot : in t_mot) return t_opcode2word;


-- fonction is_2word_operation : teste si un opcode correspondant au 
-- 		1er mot d'une instruction ncessite la lecture d'un 2e mot
-- paramtre opcode : le code  tester
-- retour : vrai si on a un opcode de type OP2_FAMILY1 ou OP2_FAMILY2
function is_2word_operation(opcode : in t_opcode1word) return boolean;

-- fonction nb_word_to_read1 : nombre de mots  lire pour passer  l'instruction suivante
-- paramtre opcode : le code  tester sur un seul mot
-- retour : nombre de mots  passer en mmoire principale pour aller  l'instruction suivante
function nb_word_to_read1(opcode : in t_opcode1word) return natural;

end pack_cpu;

package body pack_cpu is

function decode_1st_word(mot : in t_mot) return t_opcode1word is
begin
	if mot = "0000" then return O_READ;
	elsif mot(3 downto 2) = "00" then return LD_FROM_RJ;
	elsif mot = "0100" then	return LD_CSTE;
	elsif mot(3 downto 2) = "01" then return LD_TO_RJ;
	elsif mot = "1000" then	return JRC;
	elsif mot = "1001" then	return JRNC;
	elsif mot = "1010" then	return JRZ;
	elsif mot = "1011" then	return JRNZ;
	elsif mot = "1100" then	return JR;
	elsif mot = "1101" then	return DJNZ;
	elsif mot = "1110" then return OP2_FAMILY1;
	else return OP2_FAMILY2; -- code 1111
	end if;
	-- TODO : en fait il faudrait tester aussi les cas avec
	-- des valeurs diffrentes de 0/1 : X,Z,U,...
end;	

function decode_2nd_word(opcode1 : in t_opcode1word ; mot : in t_mot) return t_opcode2word is
begin
	case opcode1 is
		when OP2_FAMILY1 =>
			case mot(3 downto 2) is
				when "00" => return O_ADD;
				when "01" => return O_ADC;
				when "10" => return O_MINUS;
				when "11" => return O_MULT;
				when others => return ERROR;
			end case;
		when OP2_FAMILY2 =>
			case mot(3 downto 2) is
				when "00" => return O_DIV;
				when "01" => return O_MOD;
				when "10" => return INC;
				when others =>
					if mot = "1111" then return O_WRITE;
					else return ERROR;
					end if;
			end case;
		when others => return ERROR;
	end case;	
end;

function is_2word_operation(opcode : in t_opcode1word) return boolean is
begin
	return (opcode = OP2_FAMILY1) or (opcode = OP2_FAMILY2);
end;

function is_jump_operation(opcode : in t_opcode1word) return boolean is
begin
	return (opcode = JRC) or (opcode = JRNC) or (opcode = JRZ) or
		(opcode = JRNZ) or (opcode = JR) or (opcode = DJNZ);
end;


function nb_word_to_read1(opcode : in t_opcode1word) return natural is
begin
	if opcode = O_READ or opcode = LD_TO_RJ or opcode = LD_FROM_RJ then return 0;
	elsif is_jump_operation(opcode) then return 3;
	elsif opcode = LD_CSTE then return 1;
	else
		assert true 
		REPORT "Fetch-Decode step not finished"
		SEVERITY FAILURE;
		--
		return 0;
	end if;
end;


end pack_cpu;


